/**
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_spi.h"
#include "stm32f4xx_rcc.h"

#include "spi1.h"

/** @addtogroup SPI1
  * @{
  */  

void SPI1_LowLevel_DeInit(void);
void SPI1_LowLevel_Init(void);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  DeInitializes the peripherals used by the SPI1 driver.
  * @param  None
  * @retval None
  */
void SPI1_DeInit(void)
{
  SPI1_LowLevel_DeInit();
}

/**
  * @brief  Initializes the peripherals used by the SPI1 driver.
  * @param  None
  * @retval None
  */
void SPI1_Init(uint16_t prescaler)
{
  SPI_InitTypeDef  SPI_InitStructure;

  SPI1_LowLevel_Init();
    
  /*!< SPI configuration */
  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = prescaler;
  // Speeds for APB2 running at 84MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_256; // 0.34MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_128; // 0.68MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_64; // 1.35MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_32; // 2.75MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_16; // 5.5MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8; // 10.5MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4; // 21MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2; // 42MHz

  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  SPI_Init(SPI1, &SPI_InitStructure);

  /*!< Enable the sSPI2  */
  SPI_Cmd(SPI1, ENABLE);
}


/**
  * @brief  Sends data through the SPI interface and return nothing
  *         from the SPI bus.
  * @param  data: data to send.
  */
void SPI1_Send(uint16_t data)
{
	/*!< Loop while DR register in not empty */
	while ((SPI1->SR & SPI_I2S_FLAG_TXE) == RESET);

	SPI1->DR = data;

	// after last data send, use SPI1_WaitSend() to be sure transfer has finished
}

/**
  * @brief  Wait until the last transfer has finished
  * @retval The value of the received byte (throw away;) ).
  */
uint16_t SPI1_WaitSend(void)
{
	/*!< Loop while DR register in not empty */
	while ((SPI1->SR & SPI_I2S_FLAG_TXE) == RESET);

	/*!< Wait for transfer finished */
	// hk why can i see 3 SCK cycles AFTER busy is high?
	while ((SPI1->SR & SPI_I2S_FLAG_BSY) == SET);
	return SPI1->DR; // this read resets RXNE flag
}

/**
  * @brief  Sends data through the SPI interface and return the data received
  *         from the SPI bus.
  * @param  data: data to send.
  * @retval The value of the received data.
  */
uint16_t SPI1_SendReceive(uint16_t data)
{
	/*!< Loop while DR register in not empty */
	// hk we do not need this if we can be sure last transmission ends before
	while ((SPI1->SR & SPI_I2S_FLAG_TXE) == RESET);

	/*!< Send data through the SPI2 peripheral */
	SPI1->DR = data;

	/*!< Wait to receive data */
	while ((SPI1->SR & SPI_I2S_FLAG_RXNE) == RESET);

	/*!< Return the data read from the SPI bus */
	// hk there is a 250ns pause between transfers.
	// hk wieso braucht das hier verkackte 250ns ?

	// Wenn man nur sendet und RXNE nicht abfragt bzw DR abholt, dann ist hier keine Lcke!
	// Dann kann man aber das Ende der bertragung nicht mehr feststellen, weil das
	// scheiss Busy Flag nicht das tut was es soll.

	return SPI1->DR;
}

/**
  * @brief  Initializes the peripherals used by the SPI1 driver.
  * @param  None
  * @retval None
  */
void SPI1_LowLevel_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /*!< Enable the SPI clock */
  SPI1_CLK_INIT(SPI1_CLK, ENABLE);

  /*!< Enable GPIO clocks */
  RCC_AHB1PeriphClockCmd(SPI1_SCK_GPIO_CLK | SPI1_MISO_GPIO_CLK | SPI1_MOSI_GPIO_CLK , ENABLE);
  
  /*!< SPI pins configuration *************************************************/

  /*!< Connect SPI pins to AF5 */  
  GPIO_PinAFConfig(SPI1_SCK_GPIO_PORT, SPI1_SCK_SOURCE, SPI1_SCK_AF);
  GPIO_PinAFConfig(SPI1_MISO_GPIO_PORT, SPI1_MISO_SOURCE, SPI1_MISO_AF);
  GPIO_PinAFConfig(SPI1_MOSI_GPIO_PORT, SPI1_MOSI_SOURCE, SPI1_MOSI_AF);

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
        
  /*!< SPI SCK pin configuration */
  GPIO_InitStructure.GPIO_Pin = SPI1_SCK_PIN;
  GPIO_Init(SPI1_SCK_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MOSI pin configuration */
  GPIO_InitStructure.GPIO_Pin =  SPI1_MOSI_PIN;
  GPIO_Init(SPI1_MOSI_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MISO pin configuration */
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Pin =  SPI1_MISO_PIN;
  GPIO_Init(SPI1_MISO_GPIO_PORT, &GPIO_InitStructure);
}

/**
  * @brief  DeInitializes the peripherals used by the SPI1 driver.
  * @param  None
  * @retval None
  */
void SPI1_LowLevel_DeInit(void)
{
//  GPIO_InitTypeDef GPIO_InitStructure;

  /*!< Disable the SPI1  ************************************************/
  SPI_Cmd(SPI1, DISABLE);
  
  /*!< DeInitializes the SPI1 *******************************************/
  SPI_I2S_DeInit(SPI1);
  
  /*!< SPI1 Periph clock disable ****************************************/
//  SPI1_CLK_INIT(SPI1_CLK, DISABLE);
      
  /*!< Configure all pins used by the SPI as input floating *******************/
//  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
//  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
//
//  GPIO_InitStructure.GPIO_Pin = SPI1_SCK_PIN;
//  GPIO_Init(SPI1_SCK_GPIO_PORT, &GPIO_InitStructure);
//
//  GPIO_InitStructure.GPIO_Pin = SPI1_MISO_PIN;
//  GPIO_Init(SPI1_MISO_GPIO_PORT, &GPIO_InitStructure);
//
//  GPIO_InitStructure.GPIO_Pin = SPI1_MOSI_PIN;
//  GPIO_Init(SPI1_MOSI_GPIO_PORT, &GPIO_InitStructure);
}

/**
  * @brief  Change speed of SPI interface
  * @param  New prescaler
  * @retval Old prescaler settings
  */
uint16_t SPI1_ChangeSpeed(uint16_t prescaler)
{
	uint16_t tmp_cr1, last_prescaler;

	// Disable SPI1
    SPI1->CR1 &= (uint16_t)~((uint16_t)SPI_CR1_SPE);

    // store old settings
	tmp_cr1 = SPI1->CR1;

	// mask old prescaler value
	last_prescaler = tmp_cr1 & (uint16_t)0x0038;

	// zero prescaler bits
	tmp_cr1 &= (uint16_t)0xFFC7;

	// set new prescaler bits
	tmp_cr1 |= prescaler;

	// Enable SPI1 with new prescaler
	SPI1->CR1 = tmp_cr1 | SPI_CR1_SPE;

 	return last_prescaler;
}

/**
  * @brief  Change SPI datasize from 8 bit to 16 bit or vice versa
  * @param  new datasize
  * @retval Old datasize settings
  */
uint16_t SPI1_ChangeDatasize(uint16_t datasize)
{
	uint16_t tmp_cr1, last_datasize;

	// Disable SPI1
    SPI1->CR1 &= (uint16_t)~((uint16_t)SPI_CR1_SPE);

    // store old settings
	tmp_cr1 = SPI1->CR1;

	// mask old datasize value
	last_datasize = tmp_cr1 & (uint16_t)0x0800;

	// zero datasize bits
	tmp_cr1 &= (uint16_t)0xF7FF;

	// set new datasize bits
	tmp_cr1 |= datasize;

	// Enable SPI1 with new datasize
	SPI1->CR1 = tmp_cr1 | SPI_CR1_SPE;

 	return last_datasize;
}

/**
  * @}
  */

/**
  * @}
  */

