/**
  ******************************************************************************
  * @file    main.c
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stdint.h"

#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_spi.h"

#include "spi1.h"
#include "spi2.h"
#include "leds.h"

#define SPI1_8BIT				0
#define SPI1_16BIT				1
#define SPI2_8BIT				0
#define SPI2_16BIT				0

#define SPI_SEND_ONLY			1

void Delay(__IO uint32_t nCount);

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void)
{
	// Discovery Board LEDs
	LEDs_Init();

	// Initialize SPI interfaces
	SPI1_Init(SPI_BaudRatePrescaler_2);
	SPI2_Init(SPI_BaudRatePrescaler_2);

#if SPI1_8BIT
	SPI1_ChangeDatasize(SPI_DataSize_8b);
#else
	SPI1_ChangeDatasize(SPI_DataSize_16b);
#endif

#if SPI2_8BIT
	SPI2_ChangeDatasize(SPI_DataSize_8b);
#else
	SPI2_ChangeDatasize(SPI_DataSize_16b);
#endif

	while (1)
	{
		LED_ORANGE_ON;
#if SPI_SEND_ONLY
		SPI1_Send(0xFFFF);
		SPI1_Send(0xFFFF);
		SPI1_WaitSend();
#else
		SPI1_SendReceive(0xFFFF);
		SPI1_SendReceive(0xFFFF);
#endif
		LED_ORANGE_OFF;
	}
}


/**
  * @brief  Delay Function.
  * @param  nCount:specifies the Delay time length.
  * @retval None
  */
void Delay(__IO uint32_t nCount)
{
  while(nCount--)
  {
  }
}

